#!/bin/bash
set -e

# This script will build the training corpus, train a machine translation
#  baseline model, and evaluate it on the test set. It can be run simply
#  using:
#   nohup ./process.sh &> process.log &
#  You can follow the process of the training by using
#   tail -f process.log
#
# It requires the following tools be installed and accessible
#  For building data -->
#    KyTea:  http://www.phontron.com/kytea/  (ver. 0.2.0 or higher)
#    MT scripts:  http://homepages.inf.ed.ac.uk/jschroe1/how-to/scripts.tgz
#  For the MT system -->
#    GIZA++: http://giza-pp.googlecode.com/
#    Moses:  http://www.statmt.org/moses/
#    SRILM:  http://www.speech.sri.com/projects/srilm/download.html
# 
#  SCRIPTS_ROOTDIR must be set to the location of Moses support scripts
#  MT_SCRIPTS must be set to the directory containing the MT helper scripts
#   and the evaluation script
#
# In order to do this, you can follow the directions of Part I here:
#  http://www.statmt.org/moses_steps.html
# then install KyTea separately.
#
# If you have any problems running this, please report them to
# Graham Neubig at by sending a mail to [his last name] at gmail.com

# 0) first set the path of all variables here
WD=`pwd`
# MT_SCRIPTS=
# SCRIPTS_ROOTDIR=
# the path to KyTea
KYTEA=`which kytea` \
    && type -P $KYTEA &>/dev/null \
    || { echo "Could not find KyTea at '$KYTEA'. Aborting." >&2; exit 1; }
# the path to Moses
MOSES=`which moses` \
    && type -P $MOSES &>/dev/null \
    || { echo "Could not find Moses at '$MOSES'. Aborting." >&2; exit 1; }
# the path to ngram-count of SRILM
NGRAM_COUNT=`which ngram-count` \
    && type -P $NGRAM_COUNT &>/dev/null \
    || { echo "Could not find SRILM at '$NGRAM_COUNT'. Aborting." >&2; exit 1; }

# 1) perform a sanity check
type -P $MT_SCRIPTS/tokenizer.perl &>/dev/null || { echo "Could not find Tokenizer at $MT_SCRIPTS/tokenizer.perl. Aborting." >&2; exit 1; }
type -P $SCRIPTS_ROOTDIR/training/clean-corpus-n.perl &>/dev/null || { echo "Could not find corpus cleaner at $SCRIPTS_ROOTDIR/training/clean-corpus-n.perl. Aborting." >&2; exit 1; }

# 2) extract the sentences
[[ -e data ]] && rm -r data
mkdir data
for f in BDS BLD CLT EPR FML GNM HST LTT PNM RLW ROD SAT SCL SNT TTL; do
    echo "Extracting $f";
    [[ -e data/$f ]] || mkdir data/$f
    for g in orig/$f/*; do
        g1=`basename $g .xml`;
        echo "cat $g | script/extract-sentences.pl data/$f/$g1"
        cat $g | script/extract-sentences.pl data/$f/$g1
        if [[ $g1 = "${f}00004" ]]; then
            g2="test"
        elif [[ $g1 = "${f}00002" ]]; then
            g2="dev"
        elif [[ $g1 = "${f}00003" ]]; then
            g2="tune"
        else
            g2="train"
        fi
        cat data/$f/$g1.en >> data/$f/$f-$g2.en
        cat data/$f/$g1.ja >> data/$f/$f-$g2.ja
    done
done
[[ -e data/orig ]] && rm -r data/orig
mkdir data/orig
for f in tune dev test train; do
    echo "Making $f Set"
    cat data/*/*-$f.en >> data/orig/kyoto-$f.en
    cat data/*/*-$f.ja >> data/orig/kyoto-$f.ja
done

# 3) tokenize, clean, and lowercase the corpus
echo "Tokenizing"
[[ -e data/tok ]] && rm -r data/tok; mkdir data/tok
for f in tune dev test train; do
    echo " $f"
    $MT_SCRIPTS/tokenizer.perl en < data/orig/kyoto-$f.en > data/tok/kyoto-$f.en
    $KYTEA -model script/kytea-kyoto.mod < data/orig/kyoto-$f.ja > data/tok/kyoto-$f.ja
done
echo "Cleaning"
$SCRIPTS_ROOTDIR/training/clean-corpus-n.perl data/tok/kyoto-train ja en data/tok/kyoto-train.cln 1 40
echo "Lowercasing"
[[ -e data/low ]] && rm -r data/low; mkdir data/low
for f in data/tok/*; do
    echo " $f"
    $MT_SCRIPTS/lowercase.perl < $f > data/low/`basename $f`;
done

# 4) train the language models
[[ -e lm ]] && rm -r lm; mkdir lm
for f in en ja; do
    echo "Training $f LM"
    $NGRAM_COUNT -order 5 -interpolate -kndiscount -text data/low/kyoto-train.cln.$f -lm lm/kyoto-train.$f
done

# 5) train moses
for f in ja en; do
    if [[ $f = "ja" ]]; then e="en"; else e="ja"; fi
    [[ -e $f-$e ]] && rm -r $f-$e; mkdir $f-$e
    echo "Training Moses for $f-$e"
$SCRIPTS_ROOTDIR/training/train-model.perl -scripts-root-dir $SCRIPTS_ROOTDIR -root-dir $f-$e -corpus data/low/kyoto-train.cln -f $f -e $e -alignment grow-diag-final-and -reordering msd-bidirectional-fe -lm 0:5:$WD/lm/kyoto-train.$e:0
done

# 6) do mert
for f in ja en; do
    if [[ $f = "ja" ]]; then e="en"; else e="ja"; fi
    echo "Doing MERT for $f-$e"
    $SCRIPTS_ROOTDIR/training/mert-moses.pl data/low/kyoto-tune.$f data/low/kyoto-tune.$e $MOSES $f-$e/model/moses.ini --working-dir $f-$e/tuning --rootdir $SCRIPTS_ROOTDIR
done

# 7) test
for f in ja en; do
    if [[ $f = "ja" ]]; then e="en"; else e="ja"; fi
    [[ -e $f-$e/evaluation ]] || mkdir $f-$e/evaluation
    if [[ $f = "ja" ]]; then 
        echo "Training English recaser"
        [[ -e $f-$e/evaluation/recaser ]] || mkdir $f-$e/evaluation/recaser
        echo "$SCRIPTS_ROOTDIR/recaser/train-recaser.perl -train-script $SCRIPTS_ROOTDIR/training/train-model.perl -ngram-count $NGRAM_COUNT -corpus data/tok/kyoto-train.$e -dir $WD/$f-$e/evaluation/recaser -scripts-root-dir $SCRIPTS_ROOTDIR"
        $SCRIPTS_ROOTDIR/recaser/train-recaser.perl -train-script $SCRIPTS_ROOTDIR/training/train-model.perl -ngram-count $NGRAM_COUNT -corpus data/tok/kyoto-train.$e -dir $WD/$f-$e/evaluation/recaser -scripts-root-dir $SCRIPTS_ROOTDIR
    fi
    for t in dev test; do
        echo "Testing and evaluating for $f-$e, $t"
        # filter
        echo "$SCRIPTS_ROOTDIR/training/filter-model-given-input.pl $f-$e/evaluation/filtered.$t $f-$e/tuning/moses.ini data/low/kyoto-$t.$f"
        $SCRIPTS_ROOTDIR/training/filter-model-given-input.pl $f-$e/evaluation/filtered.$t $f-$e/tuning/moses.ini data/low/kyoto-$t.$f
        # translate
        echo "$MOSES -config $f-$e/evaluation/filtered.$t/moses.ini < data/low/kyoto-$t.$f 1> $f-$e/evaluation/$t.out 2> $f-$e/evaluation/$t.err"
        $MOSES -config $f-$e/evaluation/filtered.$t/moses.ini < data/low/kyoto-$t.$f 1> $f-$e/evaluation/$t.out 2> $f-$e/evaluation/$t.err
        # recase (if necessary)
        if [[ $e = "en" ]]; then
            echo "$SCRIPTS_ROOTDIR/recaser/recase.perl -model $f-$e/evaluation/recaser/moses.ini -in $f-$e/evaluation/$t.out -moses $MOSES > $f-$e/evaluation/$t.recased"
            $SCRIPTS_ROOTDIR/recaser/recase.perl -model $f-$e/evaluation/recaser/moses.ini -in $f-$e/evaluation/$t.out -moses $MOSES > $f-$e/evaluation/$t.recased
            echo "$MT_SCRIPTS/detokenizer.perl -l en < $f-$e/evaluation/$t.recased > $f-$e/evaluation/$t.detokenized"
            $MT_SCRIPTS/detokenizer.perl -l en < $f-$e/evaluation/$t.recased > $f-$e/evaluation/$t.detokenized
            echo "$SCRIPTS_ROOTDIR/generic/multi-bleu.perl data/orig/kyoto-$t.$e < $f-$e/evaluation/$t.detokenized > $f-$e/evaluation/$t.grade"
            script/multi-bleu.perl data/orig/kyoto-$t.$e < $f-$e/evaluation/$t.detokenized > $f-$e/evaluation/$t.grade
        else
            # note: before evaluation of Japanese files, it may be desirable to re-tokenize the file
            #  using KyTea to match the reference more closely if you are using a different
            #  segmentation standard
            echo "$SCRIPTS_ROOTDIR/generic/multi-bleu.perl data/tok/kyoto-$t.$e < $f-$e/evaluation/$t.out > $f-$e/evaluation/$t.grade"
            script/multi-bleu.perl data/tok/kyoto-$t.$e < $f-$e/evaluation/$t.out > $f-$e/evaluation/$t.grade
        fi
    done
done

grep BLEU {ja-en,en-ja}/evaluation/*.grade
